/* Simple, console-style game menu system

Copyright (C) 2009 Damian Yerrick

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef FKMENU_H
#define FKMENU_H

#include "fk.h"

typedef struct FKMenuItem {
  const char *name;
  char transition;
  char reserved[3];
} FKMenuItem;

typedef struct FKMenu {
  const char *title;
  const FKMenuItem *items;
  const unsigned char numItems;
} FKMenu;

typedef struct FKMenuState {
  const FKMenu *menu;
  unsigned char selected;
  unsigned char transition;
  unsigned char transitionTime;
  unsigned char lastVkeys;
} FKMenuState;

/**
 * Runs a menu.
 * @return true iff a transition has started
 */
bool fkMenuRun(FKMenuState *m, FKBits vkeys);

enum {
  FKMENU_MAX_ITEMS = 31,

  TRANSITION_SCROLL_DOWN = 1,
  TRANSITION_SCROLL_UP = 2,
  TRANSITION_CHANGE = 3,
  TRANSITION_ENTER_SUBMENU = 4,
  TRANSITION_EXIT_SUBMENU = 5,
  TRANSITION_OUT = 128
};

#endif
