/* FK: tile matching game

Copyright (C) 2009 Damian Yerrick

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef FK_H
#define FK_H

#include <stdint.h>
#include <stdbool.h>

#define DISPOSE(proc, obj) if (obj) { proc(obj); obj = NULL; }
#define LAZYINIT(obj, expr) if (!obj) { obj = (expr); } if (!obj) return FALSE;

// state of a klax field
#define TRACK_DEPTH 20
#define TOSS_DEPTH 8
#define PADDLE_HEIGHT 5
#define N_COLORS 8
#define N_TRACKS 5
#define BIN_HEIGHT 5

#define WILD_COLOR 0x0F
#define ATTR_MARKED 0x10

enum {
  FKGOAL_MARATHON = 0,
  FKGOAL_SECONDS,
  FKGOAL_LINES,
  FKGOAL_DIAGONALS,
  FKGOAL_TILES,
  FKGOAL_POINTS,
  FKGOAL_HORIZONTALS,
  FKGOAL_N_TYPES
};

enum {
  FKSND_TOSS = 0x0001,       // boing in klax
  FKSND_CATCH = 0x0002,
  FKSND_SHIFT = 0x0004,
  FKSND_MISS = 0x0008,       // pain in klax
  FKSND_DROP = 0x0010,
  FKSND_LINE = 0x0020,       // arpeggio in klax
  FKSND_SECTIONUP = 0x0040,  // clapping in klax
  FKSND_GAMEOVER = 0x0080    // awww in klax
};

typedef int32_t FKFixed;
typedef uint32_t FKBits;

typedef struct FKLineCheckResult {
  unsigned char verticals[3], horizontals[3], diagonals[3];
} FKLineCheckResult;

// In FK, tiles flow from the randomizer to the track to the paddle
// to the bin to the score.  FKField has five sections for these.

typedef struct FKField {

  // randomizer
  int32_t seed;
  unsigned char density;
  unsigned char densityCounter;
  unsigned char history[N_COLORS + 1];  // the +1 is for wild tile
  unsigned char nColors;
  
  // track
  FKFixed progress;
  unsigned char trackTileX[TRACK_DEPTH];
  unsigned char trackTileColor[TRACK_DEPTH];
  
  // paddle
  unsigned char paddleX;
  unsigned char paddleTileColor[PADDLE_HEIGHT];
  
  // bin
  unsigned char binTileColor[BIN_HEIGHT][N_TRACKS];
  bool needLineCheck;
  unsigned char lineClearFreezeTime;
  FKLineCheckResult lastlc;

  // scoring
  unsigned char chain;
  unsigned char lives;
  unsigned char waveSet, wave, goalType;
  uint32_t goalCount, goal;
  uint32_t score;
  uint32_t waveTime;
  uint32_t lastLCPoints;
  FKBits sounds;
} FKField;

typedef struct FKControl {
  FKBits lastKeys, newKeys;
  FKBits dasDir;
  unsigned char dasCharge;
  unsigned char mouseLastTrack;
} FKControl;

extern volatile unsigned int ticks;
extern const char *const coprNotice[];

void fkSrand(FKField *p, uint32_t value);
void fkLineCheck(FKField *p);
void fkLineClear(FKField *p);
void fkAddPoints(FKField *p, uint32_t nPoints);
void play(FKField *p);

#endif
