/* Joystick binding configurator for Allegro programs

Copyright (C) 2009 Damian Yerrick

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef JOYCONFIG_H
#define JOYCONFIG_H

#include "fk.h"
#include <allegro/joystick.h>

enum {
  VKI_MENUUP   = 0,
  VKI_MENUDOWN = 1,
  VKI_MENUOK   = 2,
  VKI_TOSS  = 3,
  VKI_SPEED = 4,
  VKI_LEFT  = 5,
  VKI_RIGHT = 6,
  VKI_DROP  = 7,
  VKI_FRET1 = 8,
  VKI_FRET2 = 9,
  VKI_FRET3 = 10,
  VKI_FRET4 = 11,
  VKI_FRET5 = 12,
  MAX_VKEYS = 13,

  VKEY_MENUUP   = 1 << VKI_MENUUP,
  VKEY_MENUDOWN = 1 << VKI_MENUDOWN,
  VKEY_MENUOK   = 1 << VKI_MENUOK,
  VKEY_TOSS  = 1 << VKI_TOSS,
  VKEY_SPEED = 1 << VKI_SPEED,
  VKEY_LEFT  = 1 << VKI_LEFT,
  VKEY_RIGHT = 1 << VKI_RIGHT,
  VKEY_DROP  = 1 << VKI_DROP,
  MAX_PLAYERS = 2
};
#define VKEY_FRET(i) (1 << ((i) + VKI_FRET1))

/**
 * The physical buttons held down.
 */
extern FKBits buttonsHeld[MAX_JOYSTICKS];

/**
 * The physical buttons held down last frame.
 */
extern FKBits lastButtonsHeld[MAX_JOYSTICKS];

/**
 * The physical buttons held down this frame that were not
 * held down last frame.
 */
extern FKBits buttonsPressed[MAX_JOYSTICKS];

/**
 * Reads all connected game controllers and updates
 * buttonsHeld, lastButtonsHeld, and buttonsPressed.
 * @return a bitfield with 1 for each controller whose
 * buttonsPressed entry is not empty
 */
FKBits readAllPads(void);

/**
 * Reads the virtual keys configured for this player.
 * Based on the current state of the keyboard and the
 * last state of readAllPads().
 * @param player the index of the player (0 to MAX_PLAYERS - 1)
 */
FKBits readVkeys(unsigned int player);

/**
 * Reads Space as Left, Enter as Right, Left Arrow as Left,
 * and Right Arrow as Right. If none of those is held down,
 * reads readVkeys(0).
 * @return as readVkeys
 */
FKBits readMenuVkeys(void);

/**
 * Names of virtual buttons.
 */
extern const char *const vkeyNames[MAX_VKEYS];

/**
 * Prompts a player to assign virtual buttons.
 * @param player the index of the player to configure
 * @return >0 for ok; 0 for cancel
 */
int setupPlayerVkeys(unsigned int player);

/**
 * Displays a list of virtual buttons held down,
 * refreshing it until Esc is pressed.
 */
void demoReadVkeys(void);

/**
 * Saves the virtual button assignment to persistent storage.
 * @return true iff it was saved
 */
bool saveVkeys(void);

/**
 * Loads the virtual button assignment from persistent storage.
 * Reasons it could not be loaded:
 * 1. the file could not be read
 * 2. the file is malformed
 * 3. pads were plugged or unplugged since last save
 * @return true iff it was loaded correctly
 */
bool loadVkeys(void);

#endif
