/* FK: tile matching game

Copyright (C) 2009 Damian Yerrick

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "drawing.h"
#include "fk.h"
#include "joyconfig.h"
#include "bfmenu.h"
#include <unistd.h>  // for getcwd
#include <time.h>  // for srand(time(0))

#define __APPNAME__ "FK Convey"
#define __APPVER__  "0.02"

const char *const coprNotice[] = {
  __APPNAME__" "__APPVER__,
  "built on "__DATE__,
  "",
  "Copyright 2009 Damian Yerrick.",
  __APPNAME__" comes with ABSOLUTELY NO",
  "WARRANTY. This is free software,",
  "and you are welcome to share it",
  "under certain conditions.",
  "See GPL.txt for details.",
  "",
  "The nineties are long gone,",
  "but who gives a dang?",
  0
};

const char *const intro_instructions[] = {
  "Catch the tiles coming down the track, and drop them",
  "into the bin. Three like tiles in a row, column, or diagonal",
  "will disappear and give you points. Chain reactions give",
  "even more points. But miss five tiles and you're out!",
  "",
  "Controls:",
  "Toss tile: Put a tile on the paddle back on the track",
  "Speed up: Move the track four times as fast",
  "Move left/right: Move the paddle one track at a time",
  "    (Hold for faster movement)",
  "Drop tile: Put a tile on the paddle into the bin",
  "Fret: Move directly to one of the five tracks",
};

extern BITMAP *back, *bg;

const FKMenuItem mainMenuItems[] = {
  {"Solo", TRANSITION_ENTER_SUBMENU},
  {"Group", TRANSITION_ENTER_SUBMENU},
  {"Setup", TRANSITION_ENTER_SUBMENU},
  {"Exit"}
};
    
const FKMenu mainMenu = {
  "FK Convey", mainMenuItems, 4
};

const FKMenuItem soloMenuItems[] = {
  {"Marathon", TRANSITION_EXIT_SUBMENU},
  {"40 Lines"},
  {"Missions"},
  {"<Back", TRANSITION_EXIT_SUBMENU}
};

const FKMenu soloMenu = {
  "Solo", soloMenuItems, 4
};

const FKMenuItem groupMenuItems[] = {
  {"Play"},
  {"<Back", TRANSITION_EXIT_SUBMENU}
};

const FKMenu groupMenu = {
  "Group", groupMenuItems, 2
};

const FKMenuItem setupMenuItems[] = {
  {"Controls", TRANSITION_ENTER_SUBMENU},
  {"<Back", TRANSITION_EXIT_SUBMENU}
};

const FKMenu setupMenu = {
  "Setup", setupMenuItems, 2
};

const FKMenuItem controlsMenuItems[] = {
  {"Player 1", TRANSITION_EXIT_SUBMENU},
  {"Player 2", TRANSITION_EXIT_SUBMENU},
  {"Test", TRANSITION_EXIT_SUBMENU},
  {"<Back", TRANSITION_EXIT_SUBMENU}
};

const FKMenu controlsMenu = {
  "Controls", controlsMenuItems, 4
};

#if 0
static const FKMenuItem demoMenuItems[] = {
  {"No Transition"},
  {"Enter Submenu", TRANSITION_ENTER_SUBMENU},
  {"Pop a Balloon"},
  {"Exit Submenu", TRANSITION_EXIT_SUBMENU},
  {"Quit"}
};

static const FKMenu demoMenu = {
  "Test", demoMenuItems, 5
};
#endif

void writeMenuMoveSound(void);

void menuDemo(void) {
  FKMenuState req = {
    &mainMenu, 0, TRANSITION_ENTER_SUBMENU, 12
  };
  bool done = false;

  clear_bitmap(screen);
  int lastTicks = ticks;
  while (!done) {
    while (!fkMenuRun(&req, readMenuVkeys())) {
      while (lastTicks >= ticks) {
        rest(5);
        vsync();
      }
      lastTicks = ticks;
    }
    if (req.menu == &mainMenu) {
      if (req.selected == 0) {
        req.menu = &soloMenu;
        req.selected = 0;
      } else if (req.selected == 1) {
        req.menu = &groupMenu;
        req.selected = 0;
      } else if (req.selected == 2) {
        req.menu = &setupMenu;
        req.selected = 0;
      } else {
        done = true;
      }
    } else if (req.menu == &soloMenu) {
      if (req.selected == 0) {
        FKField p1;
        fkSrand(&p1, time(0) ^ ticks);
        blit(back, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
        play(&p1);
        if (p1.sounds & (FKSND_GAMEOVER | FKSND_SECTIONUP)) {
          rest(500);
        }
        clear_bitmap(screen);
      } else if (req.selected < 3) {
        fkPlaySounds(FKSND_MISS);
      } else {
        req.menu = &mainMenu;
        req.selected = 0;
      }
    } else if (req.menu == &groupMenu) {
      if (req.selected == 0) {
        fkPlaySounds(FKSND_MISS);
      } else {
        req.menu = &mainMenu;
        req.selected = 1;
      }
    } else if (req.menu == &setupMenu) {
      if (req.selected == 0) {
        req.menu = &controlsMenu;
        req.selected = 0;
      } else {
        req.menu = &mainMenu;
        req.selected = 2;
      }
    } else if (req.menu == &controlsMenu) {
      if (req.selected < 2) {
        setupPlayerVkeys(req.selected);
      } else if (req.selected == 2) {
        demoReadVkeys();
      } else {
        req.menu = &setupMenu;
        req.selected = 0;
      }
    } else {
      fkPlaySounds(FKSND_MISS);
      req.menu = &mainMenu;
      req.selected = mainMenu.numItems - 1;
    }
  }
}

int alert_big(const char *line1, const char *line2, const char *line3,
              const char *button1, const char *button2, int key1, int key2) {
  const FONT *f = font;
  font = (FONT *)picken_sans;
  int pressed = alert(line1, line2, line3, button1, button2, key1, key2);
  font = (FONT *)f;
  return pressed;
}  

bool playAgain(FKField *p) {
  char line2[128];
  usprintf(line2, "Score: %lu", (unsigned long)p->score);
  return alert_big(__APPNAME__": Game Over", line2, "See you in the teens!",
                   "&Play again", "E&xit", 'p', 'x') == 1;
}

volatile unsigned int ticks = 0;

void incTicks(void) {
  ++ticks;
}
END_OF_FUNCTION(incTicks)

void cleanup(void) {
  drawing_cleanup();
}

void prtscr(void) {
  PALETTE pal;
  save_bitmap("prtscr.bmp", back, pal);
}

int main(int argc, char **argv) {
  FKField p1;
  bool full = false;
  
  for (int arg = 1; arg < argc; ++arg) {
    if (!strcmp(argv[arg], "full")) {
      full = true;
    }
  }

  if (allegro_init() < 0) {
    return EXIT_FAILURE;
  }
  install_timer();
  LOCK_FUNCTION(inc_ticks);
  LOCK_VARIABLE(ticks);
  install_int_ex(incTicks, BPM_TO_TIMER(3600));
  if (openWindow(!full) < 0) {
    allegro_message("could not open a window.\n");
    return EXIT_FAILURE;
  }
  if (!fkLoadDatafiles()) {
    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
    allegro_message("couldn't load the datafiles.\n");
    return EXIT_FAILURE;
  }
  atexit(cleanup);
  set_window_title("It is the teens and there is time for "__APPNAME__);
  install_keyboard();
  install_joystick(JOY_TYPE_AUTODETECT);
  install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL);

  install_mouse();
  show_os_cursor(MOUSE_CURSOR_ARROW);
  {
    clear_to_color(screen, makecol(0, 0, 170));
    int white = makecol(255, 255, 255);
    for (int i = 0;
         i < sizeof(intro_instructions)/sizeof(intro_instructions[0]);
         ++i) {
      textout_ex(screen, picken_sans, intro_instructions[i],
                 40, 40 + 24 * i, white, -1);
    }
  }
  show_mouse(screen);
  
  if (!loadVkeys()) {
    char cwd[PATH_MAX];
    
    getcwd(cwd, sizeof(cwd));
    alert_big("You have plugged or unplugged a joystick",
              "since you last ran "__APPNAME__".",
              "You'll need to set your controls again.",
              "OK", 0, 13, 0);
    alert_big("The wizard will walk you through each available",
              "button. You can skip a button by pressing Tab,", 
              "or you can press Esc to quit.",
              "OK", 0, 13, 0);
    if (!setupPlayerVkeys(0)) return EXIT_FAILURE;
    saveVkeys();
  } else {
    drawSubtitle("It is the teens, and there is time for...",
                 "(Press Enter to start)");
    readkey();
  }

  clear_to_color(bg, makecol(0, 0, 170));

  fkDrawBin(bg, SCREEN_W / 2, false);
  blit(bg, back, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
  
  bool done = false;
  fkSrand(&p1, time(0) ^ ticks);
  do {
    blit(back, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    play(&p1);
    if (p1.sounds & (FKSND_GAMEOVER | FKSND_SECTIONUP)) {
      rest(500);
    }
    clear_keybuf();
    done = !playAgain(&p1);
  } while (!done);
  prtscr();
  clear_to_color(screen, makecol(0, 0, 0));
  alert_big("The following is a preview of the menu",
            "system that you'll see in the next version.",
            "Hopefully it won't crash on you.",
            "OK", 0, 13, 0);
  menuDemo();

  return EXIT_SUCCESS;
} END_OF_MAIN()

